import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;

/**
 * The CellRenderer class is a display of one square within a map in the level editor's
 * JTable.
 *
 * @author 	Joshua Thompson
 * @date		01/16/02
 * @version 1.0
 *
 */
//Version History
//  01/16/02 Joshua Thompson added algorithm pseudocode
//  02/27/02 Sean Pinto wrote algorithms

public class CellRenderer extends JLabel implements TableCellRenderer
{

    /**
     * Contains the GIF file that represents a robot.
     */
    private Image robot = Toolkit.getDefaultToolkit().getImage(Constants.graphicsPath+"robotright.gif");
    /**
     * Contains the GIF file that represents a crate.
     */
    private Image crate = Toolkit.getDefaultToolkit().getImage(Constants.graphicsPath+"crate.gif");
    /**
     * Contains the GIF file that represents an empty square.
     */
    private Image empty = Toolkit.getDefaultToolkit().getImage(Constants.graphicsPath+"empty.gif");
    /**
     * Contains the GIF file that represents an incinerator.
     */
    private Image incinerator = Toolkit.getDefaultToolkit().getImage(Constants.graphicsPath+"incinerator.gif");
    /**
     * Contains the GIF file that represents a wall.
     */
    private Image wall = Toolkit.getDefaultToolkit().getImage(Constants.graphicsPath+"wall.gif");
    /**
     * The current ImageIcon being displayed by this CellRenderer.
     */
    private ImageIcon currentImage;


    /**
     * Constructor to initialize the CellRenderer.
     */
    public CellRenderer()
    {
        // INIT currentImage to empty
        currentImage = new ImageIcon(empty);
    }

    /**
     * Changes the ImageIcon being displayed by the CellRenderer.  This method is
     * required by the TableCellRenderer interface.
     *
     * @param table		 The JTable where the cell being displayed is located
     * @param value		 The Square to display
     * @param isSelected True if the cell is selected, otherwise false
     * @param hasFocus	 True if the cell has focus, otherwise false
     * @param row		 The row position of the Square
     * @param col		 The col position of the Square
     *
     * @return Returns this CellRenderer.
     */
    public Component getTableCellRendererComponent( JTable table,
            Object value, boolean isSelected, boolean hasFocus,
            int row, int column )
    {
        // INIT square to the Object value passed.
        Square square = (Square)value;

        // IF square = Square.WALL THEN
        if(square == Square.WALL)
        {
            // SET currentImage = wall
            currentImage = new ImageIcon(wall);
        }

        // ELSE IF square = Square.CRATE THEN
        else if (square == Square.CRATE)
        {
            // SET currentImage = crate
            currentImage = new ImageIcon(crate);
        }

        // ELSE IF square = Square.INCINERATOR THEN
        else if (square == Square.INCINERATOR)
        {
            // SET currentImage = incinerator
            currentImage = new ImageIcon(incinerator);
        }

        // ELSE IF square = Square.ROBOT THEN
        else if (square == Square.ROBOT)
        {
            // SET currentImage = robot
            currentImage = new ImageIcon(robot);
        }

        // ELSE
        else
        {
            // SET currentImage = empty
            currentImage = new ImageIcon(empty);
        }  // ENDIF
        // CALL method setIcon() passing current Image
        setIcon(currentImage);
        // RETURN this CellRenderer
        return this;
    }
}
