import java.io.*;

import java.net.*;

import java.util.*;

import javax.swing.JOptionPane;



/**

 * <p>The Client class is a communications interface for an individual user who

 * wants to join a multiplayer game of Sokoban.</p>

 *

 * @author Joshua Thompson and Antony George

 * @date 11/13/2001

 * @version 1.2

 *

 */

//Version History

//  11/23/01 Joshua Thompson added algorithm pseudocode

//	11/28/01 Joshua Thompson modified javadoc header and added variable

//			 descriptions

//  01/31/02 Kevin Blomseth added code

//  03/02/02 Kevin Blomseth added code for sendChat()



public class Client extends Thread

{



    /**

     * A pointer to the main Game object

     */

    private Game owner;



    /**

     * The port to connect on

     */

    private int port;



    /**

     * the user name of the client

     */

    private String userName;



    /**

     * the robot number used for identification purposes

     */

    private int robotNum;



    /**

     * the output stream to the client

     */

    private PrintWriter out;



    /**

     * the input stream from the client

     */

    private BufferedReader in;



    /**

     * the ip address of the computer to connect to

     */

    private String connectIP;



    /**

     * reading is true when accepting commands from the client and false when not

     */

    private boolean reading;



    /**

     * the socket connection to the client

     */

    private Socket socket;



    /**

     * <p>Connects to and receives input from a server.</p>

     *

     * <p><b>Pre-condition:</b> none</p>

     * <p><b>Post-condition:</b> client thread is running</p>

     */

    public void run()

    {

        // TRY

        try

        {

            // INIT socket with connectIP and port

            socket = new Socket(connectIP, port);

        }

        // CATCH a connection refused exception

        catch (ConnectException ce)

        {

            // SHOW a message dialog box informing the user that a connection could not be made

            JOptionPane.showMessageDialog(null, "Connection refused.", "Connection Error", JOptionPane.PLAIN_MESSAGE);

            // SET socket = null

            socket = null;

        }

        // CATCH an unknown host exception

        catch (UnknownHostException uhe)

        {

            // SHOW a message dialog box informing the user that a connection could not be made

            JOptionPane.showMessageDialog(null, "Could not resolve host: "+connectIP, "Connection Error", JOptionPane.PLAIN_MESSAGE);

            // SET socket = null

            socket = null;

        }

        // CATCH the default io exception

        catch (IOException ioe)

        {

            // SHOW a message dialog box informing the user that a connection could not be made

            JOptionPane.showMessageDialog(null, "Connection could not be established.", "Connection Error", JOptionPane.PLAIN_MESSAGE);

            // SET socket = null

            socket = null;

        }

        // IF socket != null THEN

        if (socket != null)

        {

            try

            {

                // INIT PrintWriter out

                out = new PrintWriter(socket.getOutputStream(), true);

                // INIT BufferedReader in

                in = new BufferedReader(new InputStreamReader(socket.getInputStream()));

                // READ welcome message from BufferedReader in

                // IF message is server full message THEN

                if (in.readLine().substring(0, 2).equals("-1"))

                {

                    // CALL method connectionClosed()

                    owner.connectionClosed();

                    // SHOW messsage box saying server is full

                    JOptionPane.showMessageDialog(null, "Server is full.", "Connection Error", JOptionPane.PLAIN_MESSAGE);

                }

                // ELSE

                else

                {

                    // SET userName to Game.getUserName()

                    userName = owner.getUserName();

                    // WRITE userName to PrintWriter out

                    out.println(userName);

                    // READ robot number from BufferedReader in

                    // SET robotNum to robot number read in

                    robotNum = Integer.parseInt(in.readLine());

                    // SET reading true

                    reading = true;

                    // WHILE reading LOOP

                    while (reading)

                    {

                        // READ command from BufferedReader in

                        // CALL method processCommand() passing the command read in

                        processCommand(in.readLine());

                        // ENDWHILE

                    }

                    // ENDIF

                }

                // CLOSE PrintWriter out





                out.close();

                // CLOSE BufferedReader in

                in.close();

                // CLOSE socket

                socket.close();

            }

            catch (IOException ioe)

            {

                // SHOW a message dialog box informing the user that a connection could not be made

                JOptionPane.showMessageDialog(null, "Connection to host was unexpectedly terminated.", "Connection Terminated", JOptionPane.PLAIN_MESSAGE);

            }

            // SHOW a message dialog box informing the user that a connection could not be made

            JOptionPane.showMessageDialog(null, "Connection to host was closed.", "Connection Closed", JOptionPane.PLAIN_MESSAGE);

        }

        // CALL method owner.connectionClosed()

        owner.connectionClosed();

    }



    /**

     * <p>Checks the command received from the server and determines what to

     * do with the information</p>

     *

     * <p><b>Pre-condition:</b> command contains the input sent from server</p>

     * <p><b>Post-condition:</b> the input has been processed and the Game

     * is updated</p>

     *

     * @param command the String recieved from the server to be processed

     *

     */

    public void processCommand(String command)

    {

        // CASE command OF

        StringTokenizer commands = new StringTokenizer(command);

        String commanded = commands.nextToken();

        // move:

        if(commanded.equals(new String("move")))

        {

            doMove(commands);

        }

        // undo:

        else if(commanded.equals(new String("undo")))

        {

            doUndo(commands);

        }

        // chat:

        else if(commanded.equals(new String("chat")))

        {

            doChat(commands);

        }

        // disconnect:

        else if(commanded.equals(new String("disconnect")))

        {

            doDisconnect(commands);

        }

        // map:

        else if(commanded.equals(new String("map")))

        {

            doMap(commands);

        }

        // restart request:

        else if (commanded.equals("resreq"))

        {

            doRequestRestart(commands);

        }

        // restart accept:

        else if (commanded.equals("restart"))

        {

            owner.doRestartMap();

        }

        // change request:

        else if (commanded.equals("chgreq"))

        {

            doRequestChange(commands);

        }

        // change accept:

        else if (commanded.equals("change"))

        {

            doAcceptChanges(commands);

        }

        // adduser:

        else if (commanded.equals("adduser"))

        {

            doAddUsers(commands);

        }

        // ENDCASE

    }



    /**

     * <p>Used by processCommand()</p>

     *

     * @param command the String recieved from process to be processed

     */

    private void doMove(StringTokenizer commands)

    {

        // READ robotNumber

        int robotNumber = Integer.parseInt(commands.nextToken());

        if (robotNumber != robotNum)

        {

            // READ direction

            String dirStr = commands.nextToken();

            Direction dir = Direction.RIGHT;

            if (dirStr.equals("UP"))

            {

                dir = Direction.UP;

            }

            else if (dirStr.equals("DOWN"))

            {

                dir = Direction.DOWN;

            }

            else if (dirStr.equals("LEFT"))

            {

                dir = Direction.LEFT;

            }

            // CALL method postMove() from owner passing robotNumber and direction

            owner.postMove(robotNumber, dir);

        }

    }



    /**

     * <p>Used by processCommand()</p>

     *

     * @param command the String recieved from process to be processed

     */

    private void doUndo(StringTokenizer commands)

    {

        // READ robotNumber

        int robotNumber = Integer.parseInt(commands.nextToken());

        if (robotNumber != robotNum)

        {

            // CALL method undoMove() from owner passing robotNumber

            owner.undoMove(robotNumber);

        }

    }



    /**

     * <p>Used by processCommand()</p>

     *

     * @param command the String recieved from process to be processed

     */

    private void doChat(StringTokenizer commands)

    {

        // READ user name

        String postname = commands.nextToken();

        String message="";

        if (!userName.equals(postname))

        {

            while(commands.hasMoreTokens())

                // READ message

                message = message + new String(commands.nextToken()) + " ";

            // CALL method postChat() from owner passing user name and message

            owner.postChat(postname, message);

        }

    }



    /**

     * <p>Used by processCommand()</p>

     *

     * @param command the String recieved from process to be processed

     */

    private void doDisconnect(StringTokenizer commands)

    {

        // READ robot number

        int robotNumber = Integer.parseInt(commands.nextToken());

        // IF robot number = robotNum THEN

        if(robotNumber == robotNum)

        {

            // SET connected false

            reading = false;

            // CALL method disconnect()

            disconnect();

        }

        // ELSE

        else

        {

            // CALL method postDisconnect() from owner

            // passing robot number

            owner.postDisconnect(robotNumber);

            // ENDIF

        }

    }



    /**

     * <p>Used by processCommand()</p>

     *

     * @param command the String recieved from process to be processed

     */

    private void doMap(StringTokenizer commands)

    {

        try

        {

            // READ level name from BufferedReader in

            String levelname = commands.nextToken("");

            levelname = levelname.substring(1, levelname.length());

            // OPEN level file with name read in

            PrintWriter hilevelname = new PrintWriter(new FileWriter(Constants.levelPath + levelname + Constants.levelExt));

            // FOR row = 0 to Constants.maxRow

            for(int row=0; row < Constants.maxRow; row++)

            {

                // READ line from BufferedReader in

                String rowdata = in.readLine();

                // TOKENIZE line into column values

                StringTokenizer rowvalue = new StringTokenizer(rowdata, " ");

                // FOR col = 0 to Constants.maxCol

                for(int col=0; col < Constants.maxCol; col++)

                {

                    // SET column value to next token

                    String colvalue = rowvalue.nextToken();

                    // WRITE column value to level file

                    hilevelname.print(colvalue + " ");

                    // ENDFOR

                }

                hilevelname.println();

                // ENDFOR

            }

            // CLOSE level file





            hilevelname.close();

            // CALL method changeLevel() within owner passing the level name

            owner.startMap(levelname);

            owner.updateDisplay();

        }

        catch (IOException ioe)

        {

            ioe.printStackTrace();

        }

    }



    /**

     * <p>Used by processCommand()</p>

     *

     * @param command the String recieved from process to be processed

     */

    private void doRequestRestart(StringTokenizer commands)

    {

        int robNum = Integer.parseInt(commands.nextToken());

        if (robNum != robotNum)

        {

            owner.requestRestart(robNum);

        }

    }



    /**

     * <p>Used by processCommand()</p>

     *

     * @param command the String recieved from process to be processed

     */

    private void doRequestChange(StringTokenizer commands)

    {

        int robNum = Integer.parseInt(commands.nextToken());

        String level = commands.nextToken("");

        if (robNum != robotNum)

        {

            owner.requestChangeLevel(level.substring(1, level.length()), robNum);

        }

    }



    /**

     * <p>Used by processCommand()</p>

     *

     * @param command the String recieved from process to be processed

     */

    private void doAcceptChanges(StringTokenizer commands)

    {

        String level = commands.nextToken("");

        level = level.substring(1, level.length());

        owner.doChangeLevel(level);

    }



    /**

     * <p>Used by processCommand()</p>

     *

     * @param command the String recieved from process to be processed

     */

    private void doAddUsers(StringTokenizer commands)

    {

        String userName = commands.nextToken();

        int robotNumber = Integer.parseInt(commands.nextToken());

        owner.postNewUser(userName, robotNumber);

    }



    /**

     * <p>Creates a move command String for the robot passed and sends the

     * command to the server.</p>

     *

     * <p><b>Pre-condition:</b> none</p>

     * <p><b>Post-condition:</b> a move command String is created and

     * sent to the server.</p>

     *

     * @param robot the robot number of the robot moving

     * @param dir the direction the robot is moving

     */

    public void sendMove(int robot, Direction dir)

    {

        // CREATE new String with move keyword, robot, and direction

        String command = "move " + robot + " " + dir.toString();

        // CALL method postCommand() with created String as parameter

        postCommand(command);

    }



    /**

     * <p>Creates an undo command String for the robot passed and sends the

     * command to to the server.</p>

     *

     * <p><b>Pre-condition:</b> none</p>

     * <p><b>Post-condition:</b> an undo command String is created and sent to the

     * server.</p>

     *

     * @param robot the robot number of the robot moving

     * @param direction the direction the robot is moving

     */

    public void sendUndo(int robot)

    {

        // CREATE new String with undo keyword and robot number passed

        // CALL method postCommand() with created String as parameter

        postCommand("undo " + robot);

    }



    /**

     * <p>Creates a chat command String for the robot passed and sends the

     * command to to the server.</p>

     *

     * <p><b>Pre-condition:</b> none</p>

     * <p><b>Post-condition:</b> a chat command String is created and sent to the

     * server.</p>

     *

     * @param user the user name of the user who sent the message

     * @param message the message sent by the user

     */

    public void sendChat(String user, String message)

    {

        // CREATE new String with chat keyword, user name and message

        // CALL method postCommand() with created String as parameter

        postCommand("chat " + user + " " + message);

    }



    /**

     * <p>Creates a restartMap command String for the robot passed and calls the

     * sendToAll() method to send the command to all connected clients.</p>

     *

     * <p><b>Pre-condition:</b> none</p>

     * <p><b>Post-condition:</b> a restart command String is created and sent to the

     * sendToAll() method.</p>

     */

    public void sendChangeLevelAccept(boolean accepted, String level)

    {

        // CREATE new String with restartMap keyword

        String command = "change " + accepted + " " + level;

        // CALL method sendToAll with created String as parameter

        postCommand(command);

    }



    /**

     * <p>Creates a restart request command String for the robot passed and

     * calls the sendToAll() method to send the command to all connected

     * clients.</p>

     *

     * <p><b>Pre-condition:</b> none</p>

     * <p><b>Post-condition:</b> a restart command String is created and sent to the

     * sendToAll() method.</p>

     */

    public void sendChangeLevelRequest(String level)

    {

        // CREATE new String with restartMap keyword

        String command = "chgreq " + level;

        // CALL method sendToAll with created String as parameter

        postCommand(command);

    }



    /**

     * <p>Creates a restartMap command String for the robot passed and calls the

     * sendToAll() method to send the command to all connected clients.</p>

     *

     * <p><b>Pre-condition:</b> none</p>

     * <p><b>Post-condition:</b> a restart command String is created and sent to the

     * sendToAll() method.</p>

     */

    public void sendRestartAccept(boolean accepted)

    {

        // CREATE new String with restartMap keyword

        String command = "restart " + accepted;

        // CALL method sendToAll with created String as parameter

        postCommand(command);

    }



    /**

     * <p>Creates a restart request command String for the robot passed and

     * calls the sendToAll() method to send the command to all connected

     * clients.</p>

     *

     * <p><b>Pre-condition:</b> none</p>

     * <p><b>Post-condition:</b> a restart command String is created and sent to the

     * sendToAll() method.</p>

     */

    public void sendRestartRequest()

    {

        // CREATE new String with restartMap keyword

        String command = "resreq " + robotNum;

        // CALL method sendToAll with created String as parameter

        postCommand(command);

    }



    /**

     * <p>Sends a disconnect message to the server.</p>

     *

     * <p><b>Pre-condition:</b> existing socket connection to a server</p>

     * <p><b>Post-condition:</b> disconnect command sent to the server; socket is closed</p>

     */

    public void disconnect()

    {

        // CREATE new String with disconnect keyword

        // CALL method postCommand() with created String as parameter

        postCommand("disconnect");

        // SET reading = false

        reading = false;

    }



    /**

     * <p>Sends the command to the server.</p>

     *

     * <p><b>Pre-condition:</b> existing socket connection to a server</p>

     * <p><b>Post-condition:</b> command parameter is sent to the server</p>

     *

     * @param command information to be sent to the server

     */

    public void postCommand(String command)

    {

        // WRITE command parameter to PrintWriter out

        out.println(command);

    }



    /**

     * <p>Client constructor</p>

     *

     * <p><b>Pre-condition:</b> none</p>

     * <p><b>Post-condition:</b> Client class created; connectIP set to value of

     * ip parameter; port set to value of port parameter</p>

     *

     * @param ip the IP address of the computer to connect to

     * @param port the port to connect to

     */

    public Client(Game owner, String ip, int port)

    {

        // SET owner = passed owner

        this.owner = owner;

        // SET connectIP = ip

        this.connectIP = ip;

        // SET port = passed port

        this.port = port;

        // SET connected = false

        reading = false;

    }



    /**

     * <p>Sets the owner attribute.</p>

     *

     * @param owner provides a link back to the game.

     */

    public void setOwner(Game owner)

    {

        // SET owner = passed owner

        this.owner = owner;

    }



    /**

     * <p>Sets the port to connect to.</p>

     *

     * @param port the port to connect to

     */

    public void setPort(int port)

    {

        // SET port = passed port

        this.port = port;

    }



    /**

     * <p>Returns the port to connect to.</p>

     *

     * @return port - the port used on the by the host computer to connect to

     */

    public int getPort()

    {

        // RETURN port

        return port;

    }



    /**

     * <p>Sets the ip address to connect to.</p>

     *

     * @param ip the IP address of the computer to connect to

     */

    public void setIP(String ip)

    {

        // SET connectIP = ip

        connectIP = ip;

    }



    /**

     * <p>Returns the ip address to connect to.</p>

     *

     * @return connectIP - the ip address of the host computer to connect to

     */

    public String getIP()

    {

        // RETURN connectIP

        return connectIP;

    }



    /**

     * <p>Returns the robot number of this client.</p>

     *

     * @return robotNum - the robot number of this client

     */

    public int getRobotNumber()

    {

        // RETURN robotNum

        return robotNum;

    }



    /**

     * <p>Returns true if a connection to a server exists and false if a

     * connection to a server doesn't exist.</p>

     *

     * @return connected - true if connected to the server, false if not

     */

    public boolean isConnected()

    {

        // RETURN reading

        return reading;

    }

}

