import java.util.*;

/**
 * CustomLevelOrder is a user defined order of the custom created levels.
 *
 * @author Joshua Thompson
 * @date 11/28/01
 * @version 1.0
 */
//Version History
//  11/28/01 Joshua Thompson added algorithm pseudocode
//  12/01/01 Joshua Thompson added java source code
//	01/12/02 Joshua Thompson modified java source code

public class CustomLevelOrder
{
    /**
     * customLevelOrder is a vector of the custom levels that have been
     * created in the level editor rearranged to the user's preference.
     */
    private Vector customLevelOrder;
    /**
     * the location in the customLevelOrder Vector of the current level
     * being played.
     */
    private int currentLevel;

    /**
     * Constructor - initializes the CustomLevelOrder attributes
     */
    public CustomLevelOrder()
    {
        customLevelOrder = new Vector();
        currentLevel = 0;
    }

    /**
     * This function allows the user to add custom level(s) to level list.
     *
     * @param location     The slot number where the added level will be saved
     * @param level        The name of the level being added
     * <BR><B>Precondition:</B>The slot number desired is empty
     * <BR><B>Postcondition:</B>The added level will be saved in the desired slot
     * with the desired name
     */
    public void insertLevel(int location, String level)
    {
        // CALL method add() from customLevelOrder vector passing location and level
        customLevelOrder.insertElementAt(level, location);
    }

    /**
     * This function allows the user to add custom level(s) to level list.
     *
     * @param level        The name of the level being added
     * <BR><B>Precondition:</B>None
     * <BR><B>Postcondition:</B>The added level will be saved at the end of the
     * customLevelOrder vector with the desired name
     */
    public void addLevel(String level)
    {
        // CALL method add() from customLevelOrder vector passing level
        customLevelOrder.addElement(level);
    }

    /**
        *  This function allows the user to remove custom level(s) from level list.
        *
        * @param location the location of the level to be removed from the list
        * <BR><B>Precondition:</B>There is a level in the customLevelOrder Vector matching the level string
        * <BR><B>Postcondition:</B>The desired level to be will be removed from the customLevelOrder Vector
        */
    public void removeLevel(int location)
    {
        // CALL method remove() from customLevelOrder vector passing location
        customLevelOrder.removeElementAt(location);
    }

    /**
     * sets the currentLevel to the index of the passed String
     *
     * @param levelName the level name of the new current level
     * <BR><B>Precondition:</B>a level with the name contained by levelName exists.
     * <BR><B>Postcondition:</B>currentLevel is set to the index of the element levelName
     */
    public void setCurrentLevel(String levelName)
    {
        currentLevel = customLevelOrder.indexOf(levelName);
    }

    /**
     * Returns the name of the current level in the list
     *
     * @return Returns the level name of the current level
     */
    public String getCurrentLevel()
    {
        return (String)customLevelOrder.elementAt(currentLevel);
    }

    /**
     * Returns a String array of the level names in the current customLevelOrder list
     *
     * @return Returns the level names of all the levels in the list
     */
    public String[] getLevelList()
    {
        String[] levels = new String[customLevelOrder.size()];
        for (int index = 0; index < levels.length; index++)
        {
            levels[index] = (String)customLevelOrder.elementAt(index);
        }
        return levels;
    }

    /**
     * Returns the name of the next level in the list
     *
     * @return Returns the level name of the next level or null if there are
     * no more levels
     */
    public String getNextLevel()
    {
        // IF currentLevel + 1 < customLevelOrder.size() THEN
        if (currentLevel + 1 < customLevelOrder.size())
        {
            // RETURN the level name of the element at the currentLevel location
            return (String)customLevelOrder.elementAt(currentLevel+1);
        }
        // ELSE
        else
        {
            // RETURN null
            return null;
            // ENDIF
        }
    }

    /**
     * Returns the name of the previous level in the list
     *
     * @return Returns the level name of the previous level or null if the current
     * level is the first level
     */
    public String getPrevLevel()
    {
        // IF currentLevel - 1 >= 0 THEN
        if (currentLevel - 1 >= 0)
        {
            // RETURN the level name of the element at the currentLevel location
            return (String)customLevelOrder.elementAt(currentLevel-1);
        }
        // ELSE
        else
        {
            // RETURN null
            return null;
            // ENDIF
        }
    }

    /**
     * Returns true if the customLevelOrder Vector is empty and false if not
     *
     * @return Returns true if the customLevelOrder Vector is empty; otherwise
     * it returns false.
     */
    public boolean isEmpty ()
    {
        // RETURN customLevelOrder.isEmpty()
        return customLevelOrder.isEmpty();
    }
}
