import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * The Intro class is the JPanel displaying the Sokoban introduction.
 *
 * @author  Joshua Thompson
 * @date    12/01/01
 * @version 1.0
 */
public class Intro extends JPanel implements KeyListener, MouseListener
{
    /**
     * Contains the GIF file with the introduction animation.
     */
    private Image intro = Toolkit.getDefaultToolkit().getImage(Constants.graphicsPath+"intro.gif");
    /**
     * The link to the Sokoban user interface.
     */
    protected SGui owner;

    /**
     * Constructor - sets the Intro panel's background to black to match the animated GIF.
     */
    public Intro(SGui owner)
    {
        // SET background = Color.black
        this.setBackground(Color.black);
        // SET owner = passed owner
        this.owner = owner;
        // LISTEN for mouse-click to skip the intro
        this.addMouseListener(this);
    }

    /**
     * Paints the JPanel, drawing the introduction animation in the middle of the window.
     *
     * @param g the graphics object used for drawing images.
     * <p><b>Pre-conditions:</b>none.</p>
     * <p><b>Post-conditions:</b>the introduction animation is shown.</p>
     */
    public void paintComponent(Graphics g)
    {
        // CALL method JPanel.paintComponent() passing the graphics object g
        super.paintComponent(g);
        // DRAW the introduction animation in the middle of the window
        g.drawImage(intro, 172, 100, this);
    }

    /**
     * mousePressed is the method required by MouseListener to respond to mouse presses.
     *
     * @param ev the MouseEvent sent by the MouseListener
     */
    public void mousePressed(MouseEvent ev)
    {
        // CALL method owner.loadMainMenu()
        owner.loadMainMenu();
        // REMOVE keylistener
        owner.removeKeyListener(this);
        // REPAINT owner
        owner.validate();
        owner.repaint();
    }

    /**
     * Empty Method: Required by the MouseListener.
     */
    public void mouseClicked(MouseEvent ev)
    {}

    /**
     * Empty Method: Required by the MouseListener.
     */
    public void mouseReleased(MouseEvent ev)
    {}

    /**
     * Empty Method: Required by the MouseListener.
     */
    public void mouseEntered(MouseEvent ev)
    {}

    /**
     * Empty Method: Required by the MouseListener.
     */
    public void mouseExited(MouseEvent ev)
    {}

    /**
     * keyPressed is the method required by KeyListener to respond to key presses.
     *
     * @param ev the KeyEvent sent by the KeyListener
     */
    public void keyPressed(KeyEvent ev)
    {
        owner.loadMainMenu();
        owner.removeKeyListener(this);
        owner.validate();
        owner.repaint();
    }

    /**
     * Empty Method: Required by the KeyListener.
     */
    public void keyReleased(KeyEvent ev)
    {}

    /**
     * Empty Method: Required by the KeyListener.
     */
    public void keyTyped(KeyEvent ev)
    {}
}
