import java.util.*;

import java.io.*;

import java.awt.*;



/**

 * The Map class represents the environment in which the robot

 * will be able to move around in. It will hold the location

 * of crates, walls, incinerators, & robots.

 *

 * @author 	Kamil Baranowski

 * @date		11/23/01

 * @version 1.4

 *

 */

//Version History

//  11/23/01 Kamil Baranowski added algorithm pseudocode

//  11/28/01 Kamil Baranowski modified headers and added variable

//			 descriptions

//  12/01/01 Joshua Thompson added java code

//	01/12/02 Joshua Thompson modified the undo method and the processMove method

//  01/28/02 Joshua Thompson modified the instantReplay method and made Map extend

//			 Observable



public class Map extends Observable

{



    /**

     * The name of the level.

     */

    protected Level level;

    /**

     * The robots is a Vector of up to 4 robots to represent each of the players in

     * multiplayer mode.

     */

    protected Vector robots;

    /**

     * A pointer to the main Game object.

     */

    private Game owner;

    /**

     * The number of crates contained in the level.

     */

    private int totalCrates;

    /**

     * The number of crates left to be incinerated.

     */

    private int cratesRemaining;

    /**

     * The timer that keeps track of the time elapsed.

     */

    private Time time;

    /**

     * The startingPoints array keeps track of the squares the robots start on.

     */

    private Point[] startingPoints;

    /**

     * The boolean to let sounds know if the game is loading or not

     */

    protected boolean loading = false;

    /**

     * Creates a new Map object with given coordinates

     * of crates, walls and incinerators.

     */

    public Map(Game owner)

    {

        // INIT robots vector

        robots = new Vector();

        // INIT level

        level = new Level();

        // INIT startingPoints

        startingPoints = new Point[4];

        // INIT time

        time = new Time();

        // SET owner = owner parameter

        this.owner = owner;

    }



    /**

     * Moves the robot in the direction passed.

     *

     * @param robotNum the robot to move

     * @param dir the direction to move

     * <br><b>Pre-conditions: none </b>

     * <br><b>Post-conditions: robot moves in the direction passed

     * <br></b>

     */

    public void moveRobot(int robotNum, Direction direction)

    {

        try

        {

            // CALL method getRobot() passing robot

            Robot robot = getRobot(robotNum);

            Move move = new Move(direction, false, false, false);

            // GET robot position from returned robot

            int row = robot.getRow();

            int col = robot.getCol();

            // GET square next to robot in the direction

            if (direction == Direction.UP)

            {

                row--;

            }

            else if (direction == Direction.DOWN)

            {

                row++;

            }

            else if (direction == Direction.LEFT)

            {

                col--;

            }

            else if (direction == Direction.RIGHT)

            {

                col++;

            }

            // IF row and col are within the map THEN

            if (row >= 0 || row <= Constants.maxRow || col >= 0 || col <= Constants.maxCol)

            {

                // IF square next to robot is EMPTY THEN

                if (level.getValueAt(row, col) == Square.EMPTY)

                {

                    // SET robot moved = true

                    move.setRobotMoved(true);

                    // SET robot's current square = EMPTY

                    level.setValueAt(Square.EMPTY, robot.getRow(), robot.getCol());

                    // SET square next to robot = ROBOT

                    level.setValueAt(Square.ROBOT, row, col);

                    // PLAY sound associated with robot moving

                    if(!loading)

                        Sound.playMoveSound();

                }

                // ELSE IF square next to robot is a CRATE THEN

                else if (level.getValueAt(row, col) == Square.CRATE)

                {

                    // GET square next to crate in direction passed

                    int nextRow = row;

                    int nextCol = col;

                    if (direction == Direction.UP)

                    {

                        nextRow--;

                    }

                    else if (direction == Direction.DOWN)

                    {

                        nextRow++;

                    }

                    else if (direction == Direction.LEFT)

                    {

                        nextCol--;

                    }

                    else if (direction == Direction.RIGHT)

                    {

                        nextCol++;

                    }

                    // IF square next to crate is within the map THEN

                    if (nextRow >= 0 || nextRow <= Constants.maxRow || nextCol >= 0 || nextCol <= Constants.maxCol)

                    {

                        // IF square next to crate is EMPTY THEN

                        if (level.getValueAt(nextRow,nextCol) == Square.EMPTY)

                        {

                            // SET crate moved = true

                            move.setCrateMoved(true);

                            // SET robot moved = true

                            move.setRobotMoved(true);

                            // SET robot's current square = EMPTY

                            level.setValueAt(Square.EMPTY, robot.getRow(), robot.getCol());

                            // SET square next to robot = ROBOT

                            level.setValueAt(Square.ROBOT, row, col);

                            // SET square next to crate = CRATE

                            level.setValueAt(Square.CRATE, nextRow, nextCol);

                            // PLAY sound associated with pushing crate

                            if(!loading)

                                Sound.playPushSound();

                        }

                        else if (level.getValueAt(nextRow,nextCol) == Square.INCINERATOR)

                        {

                            // DECREMENT cratesRemaining

                            cratesRemaining--;

                            // SET crate moved = true

                            move.setCrateMoved(true);

                            // SET crate incinerated = true

                            move.setIncinerated(true);

                            // SET robot moved = true

                            move.setRobotMoved(true);

                            // SET robot's current square = EMPTY

                            level.setValueAt(Square.EMPTY, robot.getRow(), robot.getCol());

                            // SET square next to robot = ROBOT

                            level.setValueAt(Square.ROBOT, row, col);

                            // PLAY sound associated with burning a crate

                            if(!loading)

                                Sound.playBurnSound();



                            // ENDIF

                        }

                        // ENDIF

                    }

                    // ENDIF

                }

                // ENDIF

            }

            if (move.hasRobotMoved() || robot.getDirectionFaced() != direction)

            {

                // CALL method robot.addMove() passing the new Move just created

                robot.addMove(move);

                if (cratesRemaining == 0)

                {

                    Sound.playDoneSound();

                    time.stop();

                    owner.levelCompleted();

                    // ENDIF

                }

            }

        }

        catch (ArrayIndexOutOfBoundsException aob)

        {

            // IGNORE a move outside the array

        }

    }



    /**

    * Adds a Robot to the robot vector.

    *

    * @param robot the robot to be added to the robot vector

    * <br><b>Pre-conditions: none </b>

    * <br><b>Post-conditions: a Robot is added to the robot vector

    * <br></b>

    */

    public void addRobot(Robot robot)

    {

        // IF robots.size() < 4 THEN

        if (robots.size() < 4)

        {

            // ADD robot to end of robots vector

            robots.addElement(robot);

            // ENDIF

        }

    }



    /**

    * Removes the Robot with a robotNumber equal to the passed robotNum from

    * the robot list.

    *

    * @param robot the robot number to be removed from the robot vector

    * <br><b>Pre-conditions: there are existing robots </b>

    * <br><b>Post-conditions: the robot with robotNumber = robotNum is

    * removed <br></b>

    */

    public void removeRobot(int robot)

    {

        // CALL method getRobot() passing robot

        Robot rob = getRobot(robot);

        if (rob != null)

        {

            int row = rob.getRow();

            int col = rob.getCol();

            level.setValueAt(Square.EMPTY, row, col);

            // REMOVE Robot returned from the robots vector

            robots.removeElement(rob);

        }

    }



    /**

    * Removes the Robot with a robotNumber equal to the passed robotNum from

    * the robot vector.

    *

    * @param robotNum the robot number to be found in the robots vector

    * @return the Robot with the passed robot number

    * <br><b>Pre-conditions: there are existing robots </b>

    * <br><b>Post-conditions: the robot with robotNumber = robotNum is

    * removed <br></b>

    */

    public Robot getRobot(int robotNum)

    {

        // SET index = 0

        int index = 0;

        // SET found = false

        boolean found = false;

        // INIT Robot robot

        Robot robot = null;

        // WHILE not found and index < robots.size() LOOP

        while (!found && index < robots.size())

        {

            // IF robots[index].getRobotNumber() = robotNum THEN

            robot = (Robot)robots.elementAt(index);

            if (robot.getRobotNumber() == robotNum)

            {

                // SET found true

                found = true;

                // ENDIF

            }

            // INCREMENT index





            index++;

            // ENDLOOP

        }

        // IF found THEN





        if (found)

        {

            // RETURN robots[index]

            return robot;

        }

        // ELSE

        else

        {

            // RETURN null

            return null;

            // ENDIF

        }

    }



    /**

        * Current level is reset. All crates are in orginal positions and robot

        * back at start location. All statistics are reset to 0.

        * <br><b>Pre-conditions: none </b>

        * <br><b>Post-conditions: Current level is started from the beginning. crates

        * and robot(s) are at orginal positions. stats for level are reset to zero.

        * <br></b>

        */

    public void restartLevel()

    {

        // SET cratesRemaining = totalCrates

        cratesRemaining = totalCrates;

        // CALL method reset() in time

        time.reset();

        // FOR each robot in robots array

        Robot robot;

        for (int current = 0; current < robots.size(); current++)

        {

            //     CALL method resetMoveTracker() in current robot

            robot = (Robot)robots.elementAt(current);

            robot.resetMoveTracker();

            // ENDFOR

        }

        // OPEN current level





        changeLevel(level.getLevelName());

    }



    /**

     * Shows the sequence of movements a robot has taken up to the current point

     * in the level.

     * <br><b>Pre-conditions: single-player with only 1 Robot on the robots

     * vector; the Robot has moved at least once</b>

     * <br><b>Post-conditions: the level is returned to its state prior to the

     * instant replay call

     * <br>

     */

    public void instantReplay()

    {

        // IF robots.size() = 1 and robots[0].getMoves() > 0 THEN

        Robot robot = (Robot)robots.firstElement();

        if (robots.size() == 1 && robot.getMoves() > 0)

        {



            // SET cratesRemaining = totalCrates

            cratesRemaining = totalCrates;

            // INIT Move[] moveArray to returned value of

            //     robots[0].getMoveTracker()

            Move[] moveArray = robot.getMoveTracker();

            // CALL method robots[0].resetMoveTracker()

            changeLevel(level.getLevelName());

            // NOTIFY the observers to redisplay

            this.setChanged();

            this.notifyObservers();

            // FOR current = 0 to moveArray.length

            for (int current = 0; current < moveArray.length; current++)

            {

                // CALL moveRobot() passing Move[current]

                moveRobot(robot.getRobotNumber(), moveArray[current].getDirection());

                // NOTIFY the observers to redisplay

                this.setChanged();

                this.notifyObservers(new Point(robot.getCol(), robot.getRow()));

                // CALL method Thread.sleep() passing 250 to pause for 250 ms

                try

                {

                    Thread.sleep(250);

                }

                catch (InterruptedException e)

                {

                    e.printStackTrace();

                }

                // ENDFOR

            }

            // ENDIF

        }

    }



    /**

     * Player choses a different level map to play.

     *

     * @param level    String value associated with name of the level

     * <br><b>Pre-conditions: none </b>

     * <br><b>Post-conditions: new map object is loaded for play

     * <br></b>

     */

    public void changeLevel(String newlevel)

    {

        // CLOSE current level

        // OPEN level

        try

        {

            BufferedReader in;

            in = new BufferedReader(new FileReader(Constants.levelPath+newlevel+".lvl"));

            // INIT new String input

            String input;

            // DECLARE StringTokenizer token

            StringTokenizer token;

            // INIT new int square

            int square;

            // INIT row and col = 0

            int row = 0;

            int col = 0;

            // SET totalCrates = 0

            totalCrates = 0;

            // INIT Robot robot and current robot

            Robot robot;

            int current = 0;

            // WHILE file is ready and row < Constants.maxRow LOOP

            while (in.ready() && row < Constants.maxRow)

            {

                // READ location from file

                input = in.readLine();

                // INIT new StringTokenizer token with input

                token = new StringTokenizer(input, " ");

                // SET col = 0

                col = 0;

                // WHILE token has more elements and col < Constants.maxCol LOOP

                while (token.hasMoreElements() && col < Constants.maxCol)

                {

                    square = Integer.parseInt(token.nextToken());

                    // SET board[row][col] to location from file

                    switch (square)

                    {

                        case 0: // EMPTY

                        level.setValueAt(Square.EMPTY, row, col);

                        break;

                        case 1: // WALL

                        level.setValueAt(Square.WALL, row, col);

                        break;

                        case 2: // CRATE

                        level.setValueAt(Square.CRATE, row, col);

                        totalCrates++;

                        break;

                        case 3: // INCINERATOR

                        level.setValueAt(Square.INCINERATOR, row, col);

                        break;

                        case 4: // STARTING POINT #1

                        level.setValueAt(Square.ROBOT, row, col);

                        startingPoints[0] = new Point(row, col);

                        if (robots.size() >= 1)

                        {

                            robot = (Robot)robots.elementAt(0);

                            robot.setRow(row);

                            robot.setCol(col);

                            robot.resetMoveTracker();

                        }

                        else

                        {

                            level.setValueAt(Square.EMPTY, row, col);

                        }

                        break;

                        case 5: // STARTING POINT #2

                        level.setValueAt(Square.ROBOT, row, col);

                        startingPoints[1] = new Point(row, col);

                        if (robots.size() >= 2)

                        {

                            robot = (Robot)robots.elementAt(1);

                            robot.setRow(row);

                            robot.setCol(col);

                            robot.resetMoveTracker();

                        }

                        else

                        {

                            level.setValueAt(Square.EMPTY, row, col);

                        }

                        break;

                        case 6: // STARTING POINT #3

                        level.setValueAt(Square.ROBOT, row, col);

                        startingPoints[2] = new Point(row, col);

                        if (robots.size() >= 3)

                        {

                            robot = (Robot)robots.elementAt(2);

                            robot.setRow(row);

                            robot.setCol(col);

                            robot.resetMoveTracker();

                        }

                        else

                        {

                            level.setValueAt(Square.EMPTY, row, col);

                        }

                        break;

                        case 7: // STARTING POINT #4

                        level.setValueAt(Square.ROBOT, row, col);

                        startingPoints[3] = new Point(row, col);

                        if (robots.size() == 4)

                        {

                            robot = (Robot)robots.elementAt(3);

                            robot.setRow(row);

                            robot.setCol(col);

                            robot.resetMoveTracker();

                        }

                        else

                        {

                            level.setValueAt(Square.EMPTY, row, col);

                        }

                        break;

                    }

                    // INCREMENT col

                    col++;

                    // ENDWHILE

                }

                // INCREMENT row





                row++;

                // ENDWHILE

            }

            cratesRemaining = totalCrates;

            // CALL method time.reset() to init the time

            //time.reset();

        }

        catch (IOException e)

        {

            e.printStackTrace();

        }

        // SET cratesRemaining = totalCrates

        cratesRemaining = totalCrates;

        // SET levelName = level

        level.setLevelName(newlevel);

    }



    /**

     * An algorithm will compute a score using a combination of game variables.

     * The algorithm will compute the number of crates incinerated in the current

     * level and multiply each crate by 100. The number of robot moves will be

     * subtracted from that value, and then divided by time in seconds.

     *

     * @param robot    int value associated with player's robot

     * <br><b>Pre-conditions: Level is complete, no crates are present </b>

     * <br><b>Post-conditions: adds robot's level score to robot's

     *                         cumulative game score.

     * <br></b>

     */

    public int calculateScore(int robot)

    {

        // READ time and store in seconds

        // CALL method robots[robot].calcScore() passing an int of time in seconds

        Robot rob = getRobot(robot);

        int score = rob.calcScore(time.getTime());

        // RETURN calculated score returned from calcScore() in the Robot class

        return score;

    }



    /**

     * Pauses the timer.

     * <br><b>Pre-conditions: timer is running </b>

     * <br><b>Post-conditions: timer stopped

     * <br></b>

     */

    public void pauseTimer()

    {

        // CALL class Timer method stop()

        time.stop();

    }



    /**

     * Starts the timer.

     * <br><b>Pre-conditions: timer stopped </b>

     * <br><b>Post-conditions: timer is running

     * <br></b>

     */

    public void startTimer()

    {

        // CALL class Timer method start()

        time.start();

    }



    /**

     * Sets the timer to the value passed.

     */

    public void setTimer(int newTime)

    {

        // CALL method Time.setTime() passing newTime

        time.setTime(newTime);

    }



    /**

     * Returns true if the map is currently paused, false otherwise.

     *

     * @return returns whether the map is paused or not

     */

    public boolean isPaused()

    {

        // RETURN time.isPaused()

        return time.isPaused();

    }



    /**

     * Updates the board to reflect the state before the robot's last move.

     *

     * @param robotNum the robot number who performed the undo

     * <BR><B>Precondition:</B>MoveTracker is not empty

     * <BR><B>Postcondition:</B>the board is updated to reflect the state of

     * the board before the robot's last move

     */

    public void undoMove(int robotNum)

    {

        // GET robot associated with robotNum

        Robot robot = getRobot(robotNum);

        // INIT dir = robot.getDirectionFaced()

        Direction dir = robot.getDirectionFaced();

        // IF robot.getMoves() > 0 THEN

        if (robot.getMoves() >= 1)

        {

            // INIT row and column adjustment variables

            int row = robot.getRow();

            int col = robot.getCol();

            int rowAdj = 0;

            int colAdj = 0;

            if (dir == Direction.UP)

            {

                rowAdj = -1;

            }

            else if (dir == Direction.DOWN)

            {

                rowAdj = 1;

            }

            else if (dir == Direction.LEFT)

            {

                colAdj = -1;

            }

            else if (dir == Direction.RIGHT)

            {

                colAdj = 1;

            }

            // CALL removeMove() from the Robot with number equal to robotNum

            Move undo = robot.removeMove();

            if (undo.hasRobotMoved())

            {

                if (level.getValueAt((row-rowAdj),(col-colAdj)) == Square.EMPTY)

                {

                    // GET direction from returned Move

                    //Direction dir = undo.getDirection();

                    // MOVE Robot in opposite direction of direction in returned Move

                    row = robot.getRow();

                    col = robot.getCol();

                    level.setValueAt(Square.ROBOT, row, col);

                    // IF crateMoved is true THEN

                    if (undo.hasCrateMoved())

                    {

                        //     MOVE crate next to Robot before undo in opposite direction of

                        //     direction in returned Move

                        level.setValueAt(Square.CRATE, (row+rowAdj), (col+colAdj));

                        if (undo.wasIncinerated())

                        {

                            cratesRemaining++;

                        }

                        else

                        {

                            level.setValueAt(Square.EMPTY, (row+(rowAdj*2)), (col+(colAdj*2)));

                        }

                        // ENDIF

                        // ELSE

                    }

                    else

                    {

                        level.setValueAt(Square.EMPTY, (row+rowAdj), (col+colAdj));

                        // ENDIF

                    }

                    // ENDIF

                }

                // ENDIF

            }

            // ENDIF

        }

    }



    /**

     * <p>Returns a 2-dimensional array containing the values representing

     * the current map.</p>

     *

     * @return the values representing the current map

     */

    public String[] toStringArray()

    {

        // INIT temp array of String[25]

        String[] temp = new String[Constants.maxRow];

        // TRY

        try

        {

            // INIT new BufferedReader in to read the current level file

            BufferedReader in = new BufferedReader(new FileReader(Constants.levelPath+level.getLevelName()+Constants.levelExt));

            // INIT new int row = 0

            int row = 0;

            // WHILE in.ready() and row < Constants.maxRow LOOP

            while (in.ready() && row < Constants.maxRow)

            {

                // SET temp[row] = value returned by in.readLine()

                temp[row] = in.readLine();

                // INCREMENT row

                row++;

                // ENDWHILE

            }

            // CALL method in.close()





            in.close();

            // SET row = 0

            row = 0;

            // INIT new int col = 0

            int col = 0;

            // DEFINE char[] line

            char[] line;

            // FOR index = 0 to robots.size()

            for (int index = 0; index < robots.size(); index++)

            {

                // SET row = robots[index].getRow()

                row = ((Robot)robots.elementAt(index)).getRow();

                // SET col = robots[index].getCol()

                col = ((Robot)robots.elementAt(index)).getCol();

                // SET line = value returned by temp[row].toCharArray()

                line = temp[row].toCharArray();

                // SET line[col*2] = (char)(52+index)

                line[col*2] = (char)(52+index); // 52 is ASCII '4'

                // SET temp[row] = new String() passing line

                temp[row] = new String(line);

                // ENDFOR

            }

            // ENDTRY

        }

        // CATCH IOException





        catch (IOException ioe)

        {

            // CALL method printStackTrace()

            ioe.printStackTrace();

        }

        // RETURN temp array

        return temp;

    }



    /**

     * <p>Returns a 2-dimensional array containing the Squares representing

     * the current map.</p>

     *

     * @return Returns a copy of the current map

     */

    public Square[][] getMap()

    {

        Square[][] returnmap = new Square[Constants.maxRow][Constants.maxCol];

        for(int row=0; row < Constants.maxRow; row++)

        {

            for(int col=0; col < Constants.maxCol; col++)

            {

                returnmap[row][col] = ((Square)(level.getValueAt(row, col)));

            }

        }

        return returnmap;

    }



    /**

     * <p>Returns an array of the Robots on the map.</p>

     *

     * @return the Robots playing on the current map

     */

    public Robot[] getRobots()

    {

        // INIT new Object[] temp = the value returned by robots.toArray()

        Object[] temp = robots.toArray();

        // INIT new Robot[] bots

        Robot[] bots = new Robot[temp.length];

        // FOR index = 0 to bots.length

        for (int index = 0; index < bots.length; index++)

        {

            // SET bots[index] = temp[index]

            bots[index] = (Robot)temp[index];

            // ENDFOR

        }

        // RETURN bots





        return bots;

    }



    /**

     * <p>Returns the current time spent solving the map in seconds.</p>

     *

     * @return the amount of time spent in seconds

     */

    public int getTime()

    {

        // RETURN the value returned by time.getTime()

        return time.getTime();

    }

}

