import java.util.*;

/**
 * The MoveTracker class is a data structure that contains a dynamic list of
 * the moves a robot has made.
 *
 * @author 	Joshua Thompson
 * @date	1/11/02
 * @version 1.0
 *
 */
//Version History
//  1/11/02 Joshua Thompson wrote the class definition and pseudocode
//  1/12/02 Joshua Thompson added java code

public class MoveTracker
{
    /**
     * The list of all of the moves a robot has made
     */
    private Vector moveTracker;

    /**
     * Initializes the moveTracker Vector
     */
    public MoveTracker ()
    {
        // INIT new Vector moveTracker
        moveTracker = new Vector();
    }

    /**
     * Checks to see if the moveTracker Vector is empty or not
     *
     * @return Returns true if moveTracker is empty and false if not
     */
    public boolean isEmpty()
    {
        // CALL moveTracker.isEmpty()
        return moveTracker.isEmpty();
    }

    /**
      * Resets the moveTracker to an empty value.
      *<BR><B>Precondition:</B> None
      *<BR><B>Postcondition:</B> The moveTracker vector is set to empty
      */
    public void clear()
    {
        // CALL moveTracker.clear() to empty the Vector
        moveTracker.clear();
    }

    /**
      * Adds the inputed move to the moveTracker Vector.
      *
      * @param mv	The move to be added to the vector
      * <BR><B>Precondition:</b> None
      * <BR><B>Postcondition:</B> The move was added to the moveTracer Vector
      */
    public void add (Move mv)
    {
        // CALL moveTracker.add() passing mv
        moveTracker.add(mv);
    }

    /**
     * Removes the last move from the moveTracker Vector and returns it.
     *
     * @return	Returns the move that was removed or null if the moveTracker is
     * empty
     *<BR><B>Precondition:</B> There needs to be a move on the moveTracker vector
     *<BR><B>Postcondition:</B> The move was removed from the vector
     */
    public Move removeLast()
    {
        // RETURN moveTracker.remove() passing the size of the
        //    moveTracker Vector minus 1
        return (Move)moveTracker.remove(moveTracker.size()-1);
    }

    /**
     * Returns the last move in the moveTracker Vector
     *
     * @return Returns the last move in the moveTracker or null if the
     * moveTracker is empty
     */
    public Move lastElement()
    {
        // IF moveTracker.isEmpty() returns false THEN
        if (!moveTracker.isEmpty())
        {
            // RETURN moveTracker.lastElement();
            return (Move)moveTracker.lastElement();
            // ELSE
        }
        else
        {
            // RETURN null
            return null;
            // ENDIF
        }
    }

    /**
     * Returns an array of the moves within the moveTracker Vector
     *
     * @return an array of Move
     * <BR><B>Precondition:</B> moveTracker is not empty
     * <BR><B>Postcondition:</B> an array of Move is returned
     */
    public Move[] getMoveTracker()
    {
        // INIT a temp Object array and assign it to the value returned by moveTracker.toArray()
        Object[] temp = moveTracker.toArray();
        // INIT a Moves array with same length as temp array
        Move[] moves = new Move[temp.length];
        // COPY values of temp array into the moves array
        for (int index = 0; index < moves.length; index++)
        {
            moves[index] = (Move)temp[index];
        }
        // RETURN the moves array
        return moves;
    }

}
