import javax.swing.Timer;
import java.awt.event.*;

/**
 * The Time class is the total time spent solving a map.
 *
 * @author  Joshua Thompson
 * @date    12/01/01
 * @version 1.0
 */
public class Time implements ActionListener
{
    /**
     * the current time in seconds
     */
    private int time;
    /**
     * the timer that increments the time every second
     */
    private Timer timer;

    /**
     * Constructor - initializes the timer
     */
    public Time ()
    {
        // INIT new Timer with a 1 second delay and the Time class as an ActionListener
        timer = new Timer(1000, this);
    }

    /**
     * <p>Starts the timer so that the actionPerformed method will be called once every
     * second.</p>
     *
     * <b>Pre-condition:</b>the timer has been initialized.<br>
     * <b>Post-condition:</b>the timer is started and firing ActionEvents.<br>
     */
    public void start()
    {
        // CALL method timer.start()
        timer.start();
    }

    /**
     * <p>Stops the timer from firing ActionEvents.</p>
     *
     * <b>Pre-condition:</b>the timer is started and firing ActionEvents.<br>
     * <b>Post-condition:</b>the timer is stopped.<br>
     */
    public void stop()
    {
        // CALL method timer.stop()
        timer.stop();
    }

    /**
     * <p>The total time is reset to 0.</p>
     *
     * <b>Pre-condition:</b>none.<br>
     * <b>Post-condition:</b>the time is set to 0.<br>
     */
    public void reset()
    {
        // SET time = 0
        time = 0;
        // CALL timer.stop()
        timer.stop();
        // CALL timer.start()
        timer.start();
    }

    /**
     * <p>The total time is set to the time value passed.</p>
     *
     * @param newTime the time to set the new time to
     */
    public void setTime(int newTime)
    {
        // SET time = newTime
        time = newTime;
    }

    /**
     * Returns the true if the time is paused, otherwise false.
     *
     * @return true if paused, false if running
     */
    public boolean isPaused()
    {
        // RETURN not timer.isRunning()
        return !timer.isRunning();
    }

    /**
     * <p>Returns the current time in seconds.</p>
     *
     * @return Returns an integer representing the current time in seconds.
     */
    public int getTime()
    {
        // return time
        return time;
    }

    /**
     * <p>Required for the ActionListener interface.  actionPerformed is called every
     * time the timer fires an ActionEvent, once per second.</p>
     *
     * <b>Pre-condition:</b>the timer is started and firing ActionEvents.<br>
     * <b>Post-condition:</b>the time is incremented.<br>
     */
    public void actionPerformed (ActionEvent e)
    {
        // IF the timer triggered an ActionEvent THEN
        if (e.getSource() == timer)
        {
            // INCREMENT time
            time++;
            // ENDIF
        }
    }
}
