import java.util.*;

import java.io.*;



/**

 * <p>The Tutorial is an illustration of how Sokoban is to be played.</p>

 *

 * <p>The format for a tutorial step file is:<br>

 * [Description]<br>

 * This is a step description.<br>

 * [Move]<br>

 * UP<br>

 * DOWN<br>

 * LEFT<br>

 * RIGHT<br></p>

 *

 * @author Antony George

 * @date 11/28/01

 * @version 1.0

 */

// Version History

// 03/05/02 Kevin Blomseth added code



public class Tutorial extends Map

{

    /**

     * How far along the tutorial has progressed

     */

    private int currentStep;

    /**

     * The name of the file that stores the current state of the tutorial level

     */

    private String tempLevel;

    /**

     * Storage container to hold the description of the current step

     */

    private Vector description;



    /**

     * Initializes the tutorial and loads the first step.

     */

    public Tutorial (Game owner)

    {

        super(owner);

        tempLevel = Constants.tutPath + "templevel.lvl";

        description = new Vector();

        // INIT new Robot with userName, color

        // CALL addRobot() passing the Robot

        this.addRobot(new Robot("Soko", 1));

        int input;

        // OPEN tutorial level file

        try

        {

            BufferedReader in = new BufferedReader(new FileReader(Constants.tutPath+"tutlevel.lvl"));

            // OPEN tempLevel file

            BufferedWriter outFile = new BufferedWriter(new FileWriter(tempLevel));

            // FOR row = 0 to 24

            for(int row=0; row < 24; row++)

            {

                if(in.ready())

                {

                    String inputstring = in.readLine();

                    StringTokenizer token = new StringTokenizer(inputstring, " ");

                    // FOR col = 0 to 24

                    for(int col=0; col < 24; col++)

                    {

                        // READ location from tutorial file



                        while (token.hasMoreElements())

                        {

                            input = Integer.parseInt(token.nextToken());

                            // WRITE location to tempLevel file

                            // SET board[row][col] to location from file

                            switch (input)

                            {

                                case 0: // EMPTY

                                outFile.write("0 ");

                                level.setValueAt(Square.EMPTY, row, col);

                                break;

                                case 1: // WALL

                                outFile.write("1 ");

                                level.setValueAt(Square.WALL, row, col);

                                break;

                                case 2: // CRATE

                                outFile.write("2 ");

                                level.setValueAt(Square.CRATE, row, col);

                                break;

                                case 3: // INCINERATOR

                                outFile.write("3 ");

                                level.setValueAt(Square.INCINERATOR, row, col);

                                break;

                                case 4: // STARTING POINT #1

                                outFile.write("4 ");

                                level.setValueAt(Square.ROBOT, row, col);

                                break;

                            }

                        }

                    }

                    // ENDFOR

                    outFile.newLine();

                }

                // ENDFOR

            }

            // CLOSE tutorial level file





            in.close();

            outFile.close();

            // SET currentStep = 1

            currentStep = 1;

            owner.startMap(tempLevel);

            // SET levelName to tutorial level file

            level.setLevelName(tempLevel);

            // CALL method loadStep()

            this.loadStep();



        }

        catch(IOException err)

        {

            System.out.println("I/O Exception");

        }

    }



    /**

     * <p>Loads the current step's description and moves.</p>

     *

     * <b>Pre-condition:</b>none<br>

     * <b>Post-condition:</b>description is loaded into the description

     * vector and the moves are loaded onto the moveTracker<br>

     */

    public void loadStep ()

    {

        String temp;

        try

        {

            // OPEN step file associated with currentStep

            BufferedReader in = new BufferedReader(new FileReader((".\\tutorial\\" + new Integer(currentStep)).toString() + ".tut"));

            // READ description keyword [Description]

            in.readLine();

            // CALL description.clear()

            description.clear();

            // REPEAT

            do

            {

                // READ line into temp String

                temp = in.readLine();

                // IF temp != moves keyword THEN

                if(!temp.equals("[Moves]"))

                {

                    // ADD temp String to description Vector

                    description.add(temp);

                    // END IF

                }

                // UNTIL temp = moves keyword [Moves]

            }

            while (!temp.equals("[Moves]"));

            // CALL robots[0].resetMoveTracker()

            ((Robot)robots.elementAt(0)).resetMoveTracker();

            // WHILE not end of file LOOP

            while(in.ready())

            {

                // READ line into temp String

                temp = in.readLine();

                Direction tempdir;

                if (temp.equals("UP"))

                    tempdir = Direction.UP;

                else if(temp.equals("LEFT"))

                    tempdir = Direction.LEFT;

                else if(temp.equals("DOWN"))

                    tempdir = Direction.DOWN;

                else

                    tempdir = Direction.RIGHT;

                // INIT tempMove from temp String

                Move tempMove = new Move(tempdir, false, false, true);

                // CALL method addMove() passing tempMove

                ((Robot)robots.elementAt(0)).addMove(tempMove);

                // ENDWHILE

            }

            // CLOSE current step file





            in.close();

        }

        catch(IOException err)

        {

            System.out.println("IO Exception");

            err.printStackTrace();

        }

        showStep();

    }



    /**

     * <p>Calls the instantReplay() method and then saves the board to the

     * tempLevel file.</p>

     *

     * <b>Pre-condition:</b>a tutorial step has been loaded<br>

     * <b>Post-condition:</b>the current step has been shown and the tempLevel

     * file has been stored<br>

     */

    public void showStep ()

    {

        // CALL method instantReplay() from Map parent class

        this.instantReplay();

        // CLOSE tempLevel file

        try

        {

            // OPEN tempLevel file

            BufferedWriter outFile = new BufferedWriter(new FileWriter(tempLevel));

            // FOR row = 0 to 24

            for(int row=0; row < 24; row++)

            {

                // FOR col = 0 to 24

                for(int col=0; col < 24; col++)

                {

                    // WRITE board[row][col] to tempLevel file

                    Square output = (Square)level.getValueAt(row, col);

                    // SET board[row][col] to location from file

                    if (output==Square.EMPTY)

                    {

                        outFile.write("0 ");

                    }

                    else if (output==Square.WALL)

                    {

                        outFile.write("1 ");

                    }

                    else if (output==Square.CRATE)

                    {

                        outFile.write("2 ");

                    }

                    else if (output==Square.INCINERATOR)

                    {

                        outFile.write("3 ");

                    }

                    else if (output==Square.ROBOT)

                    {

                        outFile.write("4 ");

                    }

                    // ENDFOR

                }

                // ENDFOR

            }

            outFile.newLine();

        }

        catch(IOException err)

        {

            System.out.println("I/O Exception!");

        }

    }



    /**

     * Sets the current step to the next step and then calls loadStep.

     *

     * <b>Pre-condition:</b>none<br>

     * <b>Post-condition:</b>the next step has been loaded<br>

     */

    public void getNextStep()

    {

        // INCREMENT currentStep

        currentStep++;

        // CALL method loadStep()

        this.loadStep();

    }



    /**

     * Sets the current step to the previous step and then calls loadStep.

     *

     * <br><b>Pre-condition:</b>none<br>

     * <br><b>Post-condition:</b>the previous step has been loaded<br>

     */

    public void getLastStep()

    {

        // IF currentStep > 1 THEN

        if (currentStep > 1)

        {

            // DECREMENT currentStep

            currentStep--;

            // CALL method loadStep()

            this.loadStep();

            // ENDIF

        }

    }



    /**

     * Returns an array of Strings describing the current step.

     *

     * @return a String array containing the description of the current step

     */

    public String[] getDescription()

    {

        // CALL description.toArray()

        // INIT temp String[] to returned value

        // RETURN temp String[]

        return (String[])description.toArray();

    }



    /**

     * Returns the number of the current step.

     *

     * @return the number of the current step

     */

    public int getCurrentStep()

    {

        // RETURN currentStep

        return currentStep;

    }

}

